Package: phyloseq
Version: 1.57.0
Date: 2026-02-27
Title: Handling and analysis of high-throughput microbiome census data
Description: phyloseq provides a set of classes and tools
    to facilitate the import, storage, analysis, and
    graphical display of microbiome census data.
Authors@R: c(
    person("Paul J.", "McMurdie", email = "joey711@gmail.com",
           role = c("aut", "cre")),
    person("Susan", "Holmes", email = "susan@stat.stanford.edu",
           role = "aut"),
    person("Gregory", "Jordan", role = "ctb"),
    person("Scott", "Chamberlain", role = "ctb")
    )
Encoding: UTF-8
License: AGPL-3
Imports: ade4 (>= 1.7-4), ape (>= 5.0), Biobase (>= 2.36.2),
        BiocGenerics (>= 0.22.0), biomformat (>= 1.0.0), Biostrings (>=
        2.40.0), cluster (>= 2.0.4), data.table (>= 1.10.4), foreach
        (>= 1.4.3), ggplot2 (>= 2.1.0), igraph (>= 1.0.1), methods (>=
        3.3.0), multtest (>= 2.28.0), plyr (>= 1.8.3), reshape2 (>=
        1.4.1), scales (>= 0.4.0), vegan (>= 2.5)
Depends: R (>= 3.3.0)
Suggests: BiocStyle (>= 2.4), DESeq2 (>= 1.16.1), genefilter (>= 1.58),
        knitr (>= 1.16), magrittr (>= 1.5), metagenomeSeq (>= 1.14),
        rmarkdown (>= 1.6), testthat (>= 1.0.2)
VignetteBuilder: knitr
Enhances: doParallel (>= 1.0.10)
biocViews: ImmunoOncology, Sequencing, Microbiome, Metagenomics,
        Clustering, Classification, MultipleComparison,
        GeneticVariability
URL: http://dx.plos.org/10.1371/journal.pone.0061217
BugReports: https://github.com/joey711/phyloseq/issues
Collate: 'allClasses.R' 'allPackage.R' 'allData.R' 'as-methods.R'
        'show-methods.R' 'plot-methods.R' 'extract-methods.R'
        'almostAllAccessors.R' 'otuTable-class.R' 'phyloseq-class.R'
        'taxonomyTable-class.R' 'IO-methods.R' 'merge-methods.R'
        'multtest-wrapper.R' 'ordination-methods.R'
        'transform_filter-methods.R' 'validity-methods.R'
        'assignment-methods.R' 'sampleData-class.R' 'extend_vegan.R'
        'network-methods.R' 'distance-methods.R'
        'deprecated_functions.R' 'extend_DESeq2.R' 'phylo-class.R'
        'extend_metagenomeSeq.R'
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/phyloseq
git_branch: devel
git_last_commit: c9fa481
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:21:50 UTC; biocbuild
Author: Paul J. McMurdie [aut, cre],
  Susan Holmes [aut],
  Gregory Jordan [ctb],
  Scott Chamberlain [ctb]
Maintainer: Paul J. McMurdie <joey711@gmail.com>
