# Generated by roxygen2: do not edit by hand

export("otuTable<-")
export("otu_table<-")
export("phy_tree<-")
export("sam_data<-")
export("sampleData<-")
export("sample_data<-")
export("sample_names<-")
export("speciesAreRows<-")
export("taxTab<-")
export("tax_table<-")
export("taxa_are_rows<-")
export("taxa_names<-")
export("tre<-")
export(DPCoA)
export(UniFrac)
export(access)
export(build_tax_table)
export(distance)
export(distanceMethodList)
export(estimate_richness)
export(export_env_file)
export(export_mothur_dist)
export(filter_taxa)
export(filterfunSample)
export(filterfun_sample)
export(gapstat_ord)
export(genefilterSample)
export(genefilter_sample)
export(getSamples)
export(getSpecies)
export(getTaxa)
export(getVariable)
export(get_sample)
export(get_taxa)
export(get_taxa_unique)
export(get_variable)
export(getslots.phyloseq)
export(import)
export(import_RDP_cluster)
export(import_RDP_otu)
export(import_biom)
export(import_env_file)
export(import_mothur)
export(import_mothur_dist)
export(import_pyrotagger_tab)
export(import_qiime)
export(import_qiime_otu_tax)
export(import_qiime_sampleData)
export(import_qiime_sample_data)
export(import_uparse)
export(import_usearch_uc)
export(make_network)
export(merge_phyloseq)
export(merge_phyloseq_pair)
export(merge_samples)
export(merge_species)
export(merge_taxa)
export(microbio_me_qiime)
export(mt)
export(nodeplotblank)
export(nodeplotboot)
export(nodeplotdefault)
export(nsamples)
export(nspecies)
export(ntaxa)
export(ordinate)
export(otuTable)
export(otu_table)
export(parse_taxonomy_default)
export(parse_taxonomy_greengenes)
export(parse_taxonomy_qiime)
export(phy_tree)
export(phyloseq)
export(phyloseq_to_deseq2)
export(phyloseq_to_metagenomeSeq)
export(plot_bar)
export(plot_clusgap)
export(plot_heatmap)
export(plot_net)
export(plot_network)
export(plot_ordination)
export(plot_phyloseq)
export(plot_richness)
export(plot_richness_estimates)
export(plot_scree)
export(plot_taxa_bar)
export(plot_tree)
export(prune_samples)
export(prune_species)
export(prune_taxa)
export(psmelt)
export(rank.names)
export(rank_names)
export(rarefy_even_depth)
export(read_tree)
export(read_tree_greengenes)
export(refseq)
export(rm_outlierf)
export(samData)
export(sam_data)
export(sample.names)
export(sample.variables)
export(sampleData)
export(sampleNames)
export(sampleSums)
export(sample_data)
export(sample_names)
export(sample_sums)
export(sample_variables)
export(show_mothur_cutoffs)
export(show_mothur_list_cutoffs)
export(species.names)
export(speciesAreRows)
export(speciesSums)
export(speciesarerows)
export(subset_ord_plot)
export(subset_samples)
export(subset_species)
export(subset_taxa)
export(t)
export(taxTab)
export(tax_glom)
export(tax_table)
export(taxa_are_rows)
export(taxa_names)
export(taxa_sums)
export(taxaplot)
export(taxglom)
export(taxtab)
export(threshrank)
export(threshrankfun)
export(tip_glom)
export(tipglom)
export(topf)
export(topk)
export(topp)
export(transformSampleCounts)
export(transform_sample_counts)
export(tre)
export(tree_layout)
exportClasses(dist)
exportClasses(otu_table)
exportClasses(phylo)
exportClasses(phyloseq)
exportClasses(sample_data)
exportClasses(taxonomyTable)
exportMethods("[")
exportMethods(show)
import(BiocGenerics)
import(foreach)
import(methods)
import(reshape2)
importClassesFrom(Biostrings,AAStringSet)
importClassesFrom(Biostrings,BStringSet)
importClassesFrom(Biostrings,DNAStringSet)
importClassesFrom(Biostrings,IlluminaQuality)
importClassesFrom(Biostrings,PhredQuality)
importClassesFrom(Biostrings,QualityScaledAAStringSet)
importClassesFrom(Biostrings,QualityScaledBStringSet)
importClassesFrom(Biostrings,QualityScaledDNAStringSet)
importClassesFrom(Biostrings,QualityScaledRNAStringSet)
importClassesFrom(Biostrings,QualityScaledXStringSet)
importClassesFrom(Biostrings,RNAStringSet)
importClassesFrom(Biostrings,SolexaQuality)
importClassesFrom(Biostrings,XStringQuality)
importClassesFrom(Biostrings,XStringSet)
importFrom(Biobase,AnnotatedDataFrame)
importFrom(Biostrings,readDNAStringSet)
importFrom(ade4,cailliez)
importFrom(ade4,dpcoa)
importFrom(ade4,is.euclid)
importFrom(ape,consensus)
importFrom(ape,cophenetic.phylo)
importFrom(ape,drop.tip)
importFrom(ape,is.rooted)
importFrom(ape,ladderize)
importFrom(ape,node.depth)
importFrom(ape,node.depth.edgelength)
importFrom(ape,node.height)
importFrom(ape,pcoa)
importFrom(ape,prop.part)
importFrom(ape,read.nexus)
importFrom(ape,read.tree)
importFrom(ape,reorder.phylo)
importFrom(ape,root)
importFrom(ape,write.nexus)
importFrom(biomformat,biom_data)
importFrom(biomformat,observation_metadata)
importFrom(biomformat,read_biom)
importFrom(biomformat,sample_metadata)
importFrom(cluster,agnes)
importFrom(cluster,clusGap)
importFrom(cluster,pam)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,setkey)
importFrom(data.table,setkeyv)
importFrom(data.table,setnames)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_path)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_raster)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,scale_alpha)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_size)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,scale_size_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,update_labels)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(igraph,V)
importFrom(igraph,degree)
importFrom(igraph,delete.vertices)
importFrom(igraph,get.edgelist)
importFrom(igraph,get.vertex.attribute)
importFrom(igraph,graph.adjacency)
importFrom(igraph,graph.data.frame)
importFrom(igraph,layout.auto)
importFrom(igraph,layout.circle)
importFrom(igraph,layout.fruchterman.reingold)
importFrom(igraph,layout.fruchterman.reingold.grid)
importFrom(igraph,layout.graphopt)
importFrom(igraph,layout.kamada.kawai)
importFrom(igraph,layout.lgl)
importFrom(igraph,layout.random)
importFrom(igraph,layout.reingold.tilford)
importFrom(igraph,layout.sphere)
importFrom(igraph,layout.spring)
importFrom(igraph,layout.svd)
importFrom(igraph,vcount)
importFrom(multtest,mt.maxT)
importFrom(multtest,mt.minP)
importFrom(plyr,ddply)
importFrom(plyr,is.discrete)
importFrom(plyr,ldply)
importFrom(plyr,llply)
importFrom(scales,log_trans)
importFrom(stats,p.adjust)
importFrom(stats,p.adjust.methods)
importFrom(vegan,betadiver)
importFrom(vegan,capscale)
importFrom(vegan,cca)
importFrom(vegan,decorana)
importFrom(vegan,decostand)
importFrom(vegan,designdist)
importFrom(vegan,diversity)
importFrom(vegan,estimateR)
importFrom(vegan,fisher.alpha)
importFrom(vegan,metaMDS)
importFrom(vegan,rda)
importFrom(vegan,scores)
importFrom(vegan,vegdist)
importFrom(vegan,wascores)
importFrom(vegan,wisconsin)
